@@ -19,23 +19,31 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-//! Dynamically typed value.
+//! Untyped pointer that can be cast to any struct, list, or capability type.
 
-use alloc::boxed::Box;
-use alloc::vec::Vec;
+#[cfg(feature = "alloc")]
+use alloc::{boxed::Box, vec::Vec};
 
+#[cfg(feature = "alloc")]
 use crate::capability::FromClientHook;
+#[cfg(feature = "alloc")]
 use crate::private::capability::{ClientHook, PipelineHook, PipelineOp};
-use crate::private::layout::{PointerReader, PointerBuilder};
-use crate::traits::{FromPointerReader, FromPointerBuilder, SetPointerBuilder};
+use crate::private::layout::{PointerBuilder, PointerReader};
+use crate::traits::{FromPointerBuilder, FromPointerReader, SetterInput};
 use crate::Result;
 
 #[derive(Copy, Clone)]
 pub struct Owned(());
 
-impl <'a> crate::traits::Owned<'a> for Owned {
-    type Reader = Reader<'a>;
-    type Builder = Builder<'a>;
+impl crate::traits::Owned for Owned {
+    type Reader<'a> = Reader<'a>;
+    type Builder<'a> = Builder<'a>;
+}
+
+impl crate::introspect::Introspect for Owned {
+    fn introspect() -> crate::introspect::Type {
+        crate::introspect::TypeVariant::AnyPointer.into()
+    }
 }
 
 impl crate::traits::Pipelined for Owned {
@@ -44,10 +52,10 @@ impl crate::traits::Pipelined for Owned {
 
 #[derive(Copy, Clone)]
 pub struct Reader<'a> {
-    reader: PointerReader<'a>
+    pub(crate) reader: PointerReader<'a>,
 }
 
-impl <'a> Reader<'a> {
+impl<'a> Reader<'a> {
     pub fn new(reader: PointerReader<'_>) -> Reader<'_> {
         Reader { reader }
     }
@@ -67,18 +75,20 @@ impl <'a> Reader<'a> {
         FromPointerReader::get_from_pointer(&self.reader, None)
     }
 
+    #[cfg(feature = "alloc")]
     pub fn get_as_capability<T: FromClientHook>(&self) -> Result<T> {
         Ok(FromClientHook::new(self.reader.get_capability()?))
     }
 
     //# Used by RPC system to implement pipelining. Applications
     //# generally shouldn't use this directly.
+    #[cfg(feature = "alloc")]
     pub fn get_pipelined_cap(&self, ops: &[PipelineOp]) -> Result<Box<dyn ClientHook>> {
         let mut pointer = self.reader;
 
         for op in ops {
             match *op {
-                PipelineOp::Noop =>  { }
+                PipelineOp::Noop => {}
                 PipelineOp::GetPointerField(idx) => {
                     pointer = pointer.get_struct(None)?.get_pointer_field(idx as usize);
                 }
@@ -89,8 +99,11 @@ impl <'a> Reader<'a> {
     }
 }
 
-impl <'a> FromPointerReader<'a> for Reader<'a> {
-    fn get_from_pointer(reader: &PointerReader<'a>, default: Option<&'a [crate::Word]>) -> Result<Reader<'a>> {
+impl<'a> FromPointerReader<'a> for Reader<'a> {
+    fn get_from_pointer(
+        reader: &PointerReader<'a>,
+        default: Option<&'a [crate::Word]>,
+    ) -> Result<Reader<'a>> {
         if default.is_some() {
             panic!("Unsupported: any_pointer with a default value.");
         }
@@ -98,31 +111,38 @@ impl <'a> FromPointerReader<'a> for Reader<'a> {
     }
 }
 
-impl <'a> crate::traits::SetPointerBuilder for Reader<'a> {
-    fn set_pointer_builder<'b>(mut pointer: crate::private::layout::PointerBuilder<'b>,
-                               value: Reader<'a>,
-                               canonicalize: bool) -> Result<()> {
+impl<'a> crate::traits::SetterInput<Owned> for Reader<'a> {
+    #[inline]
+    fn set_pointer_builder<'b>(
+        mut pointer: crate::private::layout::PointerBuilder<'b>,
+        value: Reader<'a>,
+        canonicalize: bool,
+    ) -> Result<()> {
         pointer.copy_from(value.reader, canonicalize)
     }
 }
 
-impl <'a> crate::traits::Imbue<'a> for Reader<'a> {
+#[cfg(feature = "alloc")]
+impl<'a> crate::traits::Imbue<'a> for Reader<'a> {
     fn imbue(&mut self, cap_table: &'a crate::private::layout::CapTable) {
-        self.reader.imbue(crate::private::layout::CapTableReader::Plain(cap_table));
+        self.reader
+            .imbue(crate::private::layout::CapTableReader::Plain(cap_table));
     }
 }
 
 pub struct Builder<'a> {
-    builder: PointerBuilder<'a>
+    builder: PointerBuilder<'a>,
 }
 
-impl <'a> Builder<'a> {
+impl<'a> Builder<'a> {
     pub fn new(builder: PointerBuilder<'a>) -> Builder<'a> {
         Builder { builder }
     }
 
     pub fn reborrow(&mut self) -> Builder<'_> {
-        Builder { builder: self.builder.reborrow() }
+        Builder {
+            builder: self.builder.reborrow(),
+        }
     }
 
     pub fn is_null(&self) -> bool {
@@ -131,14 +151,14 @@ impl <'a> Builder<'a> {
 
     /// Gets the total size of the target and all of its children. Does not count far pointer overhead.
     pub fn target_size(&self) -> Result<crate::MessageSize> {
-        self.builder.into_reader().total_size()
+        self.builder.as_reader().total_size()
     }
 
-    pub fn get_as<T : FromPointerBuilder<'a>>(self) -> Result<T> {
+    pub fn get_as<T: FromPointerBuilder<'a>>(self) -> Result<T> {
         FromPointerBuilder::get_from_pointer(self.builder, None)
     }
 
-    pub fn init_as<T : FromPointerBuilder<'a>>(self) -> T {
+    pub fn init_as<T: FromPointerBuilder<'a>>(self) -> T {
         FromPointerBuilder::init_pointer(self.builder, 0)
     }
 
@@ -146,11 +166,12 @@ impl <'a> Builder<'a> {
         FromPointerBuilder::init_pointer(self.builder, size)
     }
 
-    pub fn set_as<From: SetPointerBuilder>(self, value: From) -> Result<()> {
-        SetPointerBuilder::set_pointer_builder(self.builder, value, false)
+    pub fn set_as<T: crate::traits::Owned>(&mut self, value: impl SetterInput<T>) -> Result<()> {
+        SetterInput::set_pointer_builder(self.builder.reborrow(), value, false)
     }
 
     // XXX value should be a user client.
+    #[cfg(feature = "alloc")]
     pub fn set_as_capability(&mut self, value: Box<dyn ClientHook>) {
         self.builder.set_capability(value);
     }
@@ -161,18 +182,23 @@ impl <'a> Builder<'a> {
     }
 
     pub fn into_reader(self) -> Reader<'a> {
-        Reader { reader: self.builder.into_reader() }
+        Reader {
+            reader: self.builder.into_reader(),
+        }
     }
 }
 
-impl <'a> FromPointerBuilder<'a> for Builder<'a> {
+impl<'a> FromPointerBuilder<'a> for Builder<'a> {
     fn init_pointer(mut builder: PointerBuilder<'a>, _len: u32) -> Builder<'a> {
         if !builder.is_null() {
             builder.clear();
         }
         Builder { builder }
     }
-    fn get_from_pointer(builder: PointerBuilder<'a>, default: Option<&'a [crate::Word]>) -> Result<Builder<'a>> {
+    fn get_from_pointer(
+        builder: PointerBuilder<'a>,
+        default: Option<&'a [crate::Word]>,
+    ) -> Result<Builder<'a>> {
         if default.is_some() {
             panic!("AnyPointer defaults are unsupported")
         }
@@ -180,48 +206,88 @@ impl <'a> FromPointerBuilder<'a> for Builder<'a> {
     }
 }
 
-impl <'a> crate::traits::ImbueMut<'a> for Builder<'a> {
+#[cfg(feature = "alloc")]
+impl<'a> crate::traits::ImbueMut<'a> for Builder<'a> {
     fn imbue_mut(&mut self, cap_table: &'a mut crate::private::layout::CapTable) {
-        self.builder.imbue(crate::private::layout::CapTableBuilder::Plain(cap_table));
+        self.builder
+            .imbue(crate::private::layout::CapTableBuilder::Plain(cap_table));
     }
 }
 
 pub struct Pipeline {
     // XXX this should not be public
+    #[cfg(feature = "alloc")]
     pub hook: Box<dyn PipelineHook>,
 
+    #[cfg(feature = "alloc")]
     ops: Vec<PipelineOp>,
 }
 
 impl Pipeline {
-    pub fn new(hook: Box<dyn PipelineHook>) -> Pipeline {
-        Pipeline { hook, ops: Vec::new() }
+    #[cfg(feature = "alloc")]
+    pub fn new(hook: Box<dyn PipelineHook>) -> Self {
+        Self {
+            hook,
+            ops: Vec::new(),
+        }
     }
 
-    pub fn noop(&self) -> Pipeline {
-        Pipeline { hook: self.hook.add_ref(), ops: self.ops.clone() }
+    #[cfg(feature = "alloc")]
+    pub fn noop(&self) -> Self {
+        Self {
+            hook: self.hook.add_ref(),
+            ops: self.ops.clone(),
+        }
     }
 
-    pub fn get_pointer_field(&self, pointer_index: u16) -> Pipeline {
+    #[cfg(not(feature = "alloc"))]
+    pub fn noop(&self) -> Self {
+        Self {}
+    }
+
+    #[cfg(feature = "alloc")]
+    pub fn get_pointer_field(&self, pointer_index: u16) -> Self {
         let mut new_ops = Vec::with_capacity(self.ops.len() + 1);
         for op in &self.ops {
             new_ops.push(*op)
         }
         new_ops.push(PipelineOp::GetPointerField(pointer_index));
-        Pipeline { hook : self.hook.add_ref(), ops: new_ops }
+        Self {
+            hook: self.hook.add_ref(),
+            ops: new_ops,
+        }
     }
 
+    #[cfg(not(feature = "alloc"))]
+    pub fn get_pointer_field(&self, _pointer_index: u16) -> Self {
+        Self {}
+    }
+
+    #[cfg(feature = "alloc")]
     pub fn as_cap(&self) -> Box<dyn ClientHook> {
         self.hook.get_pipelined_cap(&self.ops)
     }
 }
 
 impl crate::capability::FromTypelessPipeline for Pipeline {
-    fn new(typeless: Pipeline) -> Pipeline {
+    fn new(typeless: Pipeline) -> Self {
         typeless
     }
 }
 
+impl<'a> From<Reader<'a>> for crate::dynamic_value::Reader<'a> {
+    fn from(a: Reader<'a>) -> crate::dynamic_value::Reader<'a> {
+        crate::dynamic_value::Reader::AnyPointer(a)
+    }
+}
+
+impl<'a> From<Builder<'a>> for crate::dynamic_value::Builder<'a> {
+    fn from(a: Builder<'a>) -> crate::dynamic_value::Builder<'a> {
+        crate::dynamic_value::Builder::AnyPointer(a)
+    }
+}
+
+#[cfg(feature = "alloc")]
 #[test]
 fn init_clears_value() {
     let mut message = crate::message::Builder::new_default();
@@ -239,7 +305,7 @@ fn init_clears_value() {
     }
 
     let mut output: Vec<u8> = Vec::new();
-    crate::serialize::write_message(&mut output, &mut message).unwrap();
+    crate::serialize::write_message(&mut output, &message).unwrap();
     assert_eq!(output.len(), 40);
     for byte in &output[8..] {
         // Everything not in the message header is zero.
