@@ -18,17 +18,18 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-use alloc::string::String;
+#[cfg(feature = "alloc")]
 use alloc::vec::Vec;
-use core::cell::RefCell;
 use core::slice;
 use core::u64;
 
-use crate::private::units::*;
-use crate::private::read_limiter::ReadLimiter;
 use crate::message;
-use crate::message::{Allocator, ReaderSegments};
-use crate::{Error, OutputSegments, Result};
+use crate::message::Allocator;
+use crate::message::ReaderSegments;
+use crate::private::read_limiter::ReadLimiter;
+use crate::private::units::*;
+use crate::OutputSegments;
+use crate::{Error, ErrorKind, Result};
 
 pub type SegmentId = u32;
 
@@ -36,7 +37,12 @@ pub trait ReaderArena {
     // return pointer to start of segment, and number of words in that segment
     fn get_segment(&self, id: u32) -> Result<(*const u8, u32)>;
 
-    fn check_offset(&self, segment_id: u32, start: *const u8, offset_in_words: i32) -> Result<*const u8>;
+    unsafe fn check_offset(
+        &self,
+        segment_id: u32,
+        start: *const u8,
+        offset_in_words: i32,
+    ) -> Result<*const u8>;
     fn contains_interval(&self, segment_id: u32, start: *const u8, size: usize) -> Result<()>;
     fn amplified_read(&self, virtual_amount: u64) -> Result<()>;
 
@@ -61,13 +67,13 @@ fn _assert_sync() {
     }
 }
 
-impl <S> ReaderArenaImpl <S> where S: ReaderSegments {
-    pub fn new(segments: S,
-               options: message::ReaderOptions)
-               -> Self
-    {
+impl<S> ReaderArenaImpl<S>
+where
+    S: ReaderSegments,
+{
+    pub fn new(segments: S, options: message::ReaderOptions) -> Self {
         let limiter = ReadLimiter::new(options.traversal_limit_in_words);
-        ReaderArenaImpl {
+        Self {
             segments,
             read_limiter: limiter,
             nesting_limit: options.nesting_limit,
@@ -79,34 +85,42 @@ impl <S> ReaderArenaImpl <S> where S: ReaderSegments {
     }
 }
 
-impl <S> ReaderArena for ReaderArenaImpl<S> where S: ReaderSegments {
+impl<S> ReaderArena for ReaderArenaImpl<S>
+where
+    S: ReaderSegments,
+{
     fn get_segment(&self, id: u32) -> Result<(*const u8, u32)> {
         match self.segments.get_segment(id) {
             Some(seg) => {
                 #[cfg(not(feature = "unaligned"))]
                 {
                     if seg.as_ptr() as usize % BYTES_PER_WORD != 0 {
-                        return Err(Error::failed(
-                            String::from("Detected unaligned segment. You must either ensure all of your \
-                                          segments are 8-byte aligned, or you must enable the \"unaligned\" \
-                                          feature in the capnp crate")))
+                        return Err(Error::from_kind(ErrorKind::UnalignedSegment));
                     }
                 }
 
                 Ok((seg.as_ptr(), (seg.len() / BYTES_PER_WORD) as u32))
             }
-            None => Err(Error::failed(format!("Invalid segment id: {}", id))),
+            None => Err(Error::from_kind(ErrorKind::InvalidSegmentId(id))),
         }
     }
 
-    fn check_offset(&self, segment_id: u32, start: *const u8, offset_in_words: i32) -> Result<*const u8> {
+    unsafe fn check_offset(
+        &self,
+        segment_id: u32,
+        start: *const u8,
+        offset_in_words: i32,
+    ) -> Result<*const u8> {
         let (segment_start, segment_len) = self.get_segment(segment_id)?;
         let this_start: usize = segment_start as usize;
         let this_size: usize = segment_len as usize * BYTES_PER_WORD;
-        let offset: i64 = offset_in_words as i64 * BYTES_PER_WORD as i64;
+        let offset: i64 = i64::from(offset_in_words) * BYTES_PER_WORD as i64;
         let start_idx = start as usize;
-        if start_idx < this_start || ((start_idx - this_start) as i64 + offset) as usize > this_size {
-            Err(Error::failed(String::from("message contained out-of-bounds pointer")))
+        if start_idx < this_start || ((start_idx - this_start) as i64 + offset) as usize > this_size
+        {
+            Err(Error::from_kind(
+                ErrorKind::MessageContainsOutOfBoundsPointer,
+            ))
         } else {
             unsafe { Ok(start.offset(offset as isize)) }
         }
@@ -120,7 +134,9 @@ impl <S> ReaderArena for ReaderArenaImpl<S> where S: ReaderSegments {
         let size = size_in_words * BYTES_PER_WORD;
 
         if !(start >= this_start && start - this_start + size <= this_size) {
-            Err(Error::failed(String::from("message contained out-of-bounds pointer")))
+            Err(Error::from_kind(
+                ErrorKind::MessageContainsOutOfBoundsPointer,
+            ))
         } else {
             self.read_limiter.can_read(size_in_words)
         }
@@ -130,96 +146,183 @@ impl <S> ReaderArena for ReaderArenaImpl<S> where S: ReaderSegments {
         self.read_limiter.can_read(virtual_amount as usize)
     }
 
-    fn nesting_limit(&self) -> i32 { self.nesting_limit }
+    fn nesting_limit(&self) -> i32 {
+        self.nesting_limit
+    }
 }
 
 pub trait BuilderArena: ReaderArena {
-    // These methods all take an immutable &self because otherwise a StructBuilder<'a>
-    // would need a `&'a mut BuilderArena` and `StructBuilder::borrow()` would
-    // have lifetime issues. (If `'a: 'b`, then a `&'a (BuilderArena + 'a)` can be
-    // converted to a `&'b (BuilderArena + 'b)`, but a `&'a mut (BuilderArena + 'a)`
-    // *cannot* be converted to a `&'b mut (BuilderArena + 'b)`. See some discussion here:
-    // https://botbot.me/mozilla/rust/2017-01-31/?msg=80228117&page=19 .)
-    fn allocate(&self, segment_id: u32, amount: WordCount32) -> Option<u32>;
-    fn allocate_anywhere(&self, amount: u32) -> (SegmentId, u32);
-    fn get_segment_mut(&self, id: u32) -> (*mut u8, u32);
+    fn allocate(&mut self, segment_id: u32, amount: WordCount32) -> Option<u32>;
+    fn allocate_anywhere(&mut self, amount: u32) -> (SegmentId, u32);
+    fn get_segment_mut(&mut self, id: u32) -> (*mut u8, u32);
 
     fn as_reader(&self) -> &dyn ReaderArena;
 }
 
+/// A wrapper around a memory segment used in building a message.
 struct BuilderSegment {
+    /// Pointer to the start of the segment.
     ptr: *mut u8,
-    capacity: u32, // in words
-    allocated: u32, // in words
+
+    /// Total number of words the segment could potentially use. That is, all
+    /// bytes from `ptr` to `ptr + (capacity * 8)` may be used in the segment.
+    capacity: u32,
+
+    /// Number of words already used in the segment.
+    allocated: u32,
 }
 
-pub struct BuilderArenaImplInner<A> where A: Allocator {
+#[cfg(feature = "alloc")]
+type BuilderSegmentArray = Vec<BuilderSegment>;
+
+#[cfg(not(feature = "alloc"))]
+#[derive(Default)]
+struct BuilderSegmentArray {
+    // In the no-alloc case, we only allow a single segment.
+    segment: Option<BuilderSegment>,
+}
+
+#[cfg(not(feature = "alloc"))]
+impl BuilderSegmentArray {
+    fn len(&self) -> usize {
+        match self.segment {
+            Some(_) => 1,
+            None => 0,
+        }
+    }
+
+    fn push(&mut self, segment: BuilderSegment) {
+        if self.segment.is_some() {
+            panic!("multiple segments are not supported in no-alloc mode")
+        }
+        self.segment = Some(segment);
+    }
+}
+
+#[cfg(not(feature = "alloc"))]
+impl core::ops::Index<usize> for BuilderSegmentArray {
+    type Output = BuilderSegment;
+
+    fn index(&self, index: usize) -> &Self::Output {
+        assert_eq!(index, 0);
+        match &self.segment {
+            Some(s) => s,
+            None => panic!("no segment"),
+        }
+    }
+}
+
+#[cfg(not(feature = "alloc"))]
+impl core::ops::IndexMut<usize> for BuilderSegmentArray {
+    fn index_mut(&mut self, index: usize) -> &mut Self::Output {
+        assert_eq!(index, 0);
+        match &mut self.segment {
+            Some(s) => s,
+            None => panic!("no segment"),
+        }
+    }
+}
+
+pub struct BuilderArenaImplInner<A>
+where
+    A: Allocator,
+{
     allocator: Option<A>, // None if has already be deallocated.
-
-    // TODO(perf): Try using smallvec to avoid heap allocations in the single-segment case?
-    segments: Vec<BuilderSegment>,
+    segments: BuilderSegmentArray,
 }
 
-pub struct BuilderArenaImpl<A> where A: Allocator {
-    inner: RefCell<BuilderArenaImplInner<A>>
+pub struct BuilderArenaImpl<A>
+where
+    A: Allocator,
+{
+    inner: BuilderArenaImplInner<A>,
 }
 
-impl <A> BuilderArenaImpl<A> where A: Allocator {
+impl<A> BuilderArenaImpl<A>
+where
+    A: Allocator,
+{
     pub fn new(allocator: A) -> Self {
-        BuilderArenaImpl {
-            inner: RefCell::new(BuilderArenaImplInner {
+        Self {
+            inner: BuilderArenaImplInner {
                 allocator: Some(allocator),
-                segments: Vec::new(),
-            }),
+                segments: Default::default(),
+            },
         }
     }
 
     /// Allocates a new segment with capacity for at least `minimum_size` words.
-    pub fn allocate_segment(&self, minimum_size: u32) -> Result<()> {
-        self.inner.borrow_mut().allocate_segment(minimum_size)
+    pub fn allocate_segment(&mut self, minimum_size: u32) -> Result<()> {
+        self.inner.allocate_segment(minimum_size)
     }
 
-    pub fn get_segments_for_output(&self) -> OutputSegments<'_> {
-        let reff = self.inner.borrow();
+    pub fn get_segments_for_output(&self) -> OutputSegments {
+        let reff = &self.inner;
         if reff.segments.len() == 1 {
             let seg = &reff.segments[0];
 
             // The user must mutably borrow the `message::Builder` to be able to modify segment memory.
             // No such borrow will be possible while `self` is still immutably borrowed from this method,
             // so returning this slice is safe.
-            let slice = unsafe { slice::from_raw_parts(seg.ptr as *const _, seg.allocated as usize * BYTES_PER_WORD) };
+            let slice = unsafe {
+                slice::from_raw_parts(seg.ptr as *const _, seg.allocated as usize * BYTES_PER_WORD)
+            };
             OutputSegments::SingleSegment([slice])
         } else {
-            let mut v = Vec::with_capacity(reff.segments.len());
-            for seg in &reff.segments {
-                // See safety argument in above branch.
-                let slice = unsafe { slice::from_raw_parts(seg.ptr as *const _, seg.allocated as usize * BYTES_PER_WORD) };
-                v.push(slice);
+            #[cfg(feature = "alloc")]
+            {
+                let mut v = Vec::with_capacity(reff.segments.len());
+                for seg in &reff.segments {
+                    // See safety argument in above branch.
+                    let slice = unsafe {
+                        slice::from_raw_parts(
+                            seg.ptr as *const _,
+                            seg.allocated as usize * BYTES_PER_WORD,
+                        )
+                    };
+                    v.push(slice);
+                }
+                OutputSegments::MultiSegment(v)
+            }
+            #[cfg(not(feature = "alloc"))]
+            {
+                panic!("invalid number of segments: {}", reff.segments.len());
             }
-            OutputSegments::MultiSegment(v)
         }
     }
 
     pub fn len(&self) -> usize {
-        self.inner.borrow().segments.len()
+        self.inner.segments.len()
     }
 
-    pub fn into_allocator(self) -> A {
-        let mut inner = self.inner.into_inner();
-        inner.deallocate_all();
-        inner.allocator.take().unwrap()
+    pub fn is_empty(&self) -> bool {
+        self.len() == 0
+    }
+
+    /// Retrieves the underlying `Allocator`, deallocating all currently-allocated
+    /// segments.
+    pub fn into_allocator(mut self) -> A {
+        self.inner.deallocate_all();
+        self.inner.allocator.take().unwrap()
     }
 }
 
-impl <A> ReaderArena for BuilderArenaImpl<A> where A: Allocator {
+impl<A> ReaderArena for BuilderArenaImpl<A>
+where
+    A: Allocator,
+{
     fn get_segment(&self, id: u32) -> Result<(*const u8, u32)> {
-        let borrow = self.inner.borrow();
-        let seg = &borrow.segments[id as usize];
+        let seg = &self.inner.segments[id as usize];
         Ok((seg.ptr, seg.allocated))
     }
 
-    fn check_offset(&self, _segment_id: u32, start: *const u8, offset_in_words: i32) -> Result<*const u8> {
-        unsafe { Ok(start.offset((offset_in_words as i64 * BYTES_PER_WORD as i64) as isize)) }
+    unsafe fn check_offset(
+        &self,
+        _segment_id: u32,
+        start: *const u8,
+        offset_in_words: i32,
+    ) -> Result<*const u8> {
+        unsafe { Ok(start.offset((i64::from(offset_in_words) * BYTES_PER_WORD as i64) as isize)) }
     }
 
     fn contains_interval(&self, _id: u32, _start: *const u8, _size: usize) -> Result<()> {
@@ -230,17 +333,26 @@ impl <A> ReaderArena for BuilderArenaImpl<A> where A: Allocator {
         Ok(())
     }
 
-    fn nesting_limit(&self) -> i32 { 0x7fffffff }
+    fn nesting_limit(&self) -> i32 {
+        0x7fffffff
+    }
 }
 
-impl <A> BuilderArenaImplInner<A> where A: Allocator {
+impl<A> BuilderArenaImplInner<A>
+where
+    A: Allocator,
+{
     /// Allocates a new segment with capacity for at least `minimum_size` words.
     fn allocate_segment(&mut self, minimum_size: WordCount32) -> Result<()> {
-        let seg = match self.allocator {
-            Some(ref mut a) => a.allocate_segment(minimum_size),
+        let seg = match &mut self.allocator {
+            Some(a) => a.allocate_segment(minimum_size),
             None => unreachable!(),
         };
-        self.segments.push(BuilderSegment { ptr: seg.0, capacity: seg.1, allocated: 0});
+        self.segments.push(BuilderSegment {
+            ptr: seg.0,
+            capacity: seg.1,
+            allocated: 0,
+        });
         Ok(())
     }
 
@@ -258,7 +370,7 @@ impl <A> BuilderArenaImplInner<A> where A: Allocator {
     fn allocate_anywhere(&mut self, amount: u32) -> (SegmentId, u32) {
         // first try the existing segments, then try allocating a new segment.
         let allocated_len = self.segments.len() as u32;
-        for segment_id in 0.. allocated_len {
+        for segment_id in 0..allocated_len {
             if let Some(idx) = self.allocate(segment_id, amount) {
                 return (segment_id, idx);
             }
@@ -267,14 +379,27 @@ impl <A> BuilderArenaImplInner<A> where A: Allocator {
         // Need to allocate a new segment.
 
         self.allocate_segment(amount).expect("allocate new segment");
-        (allocated_len,
-         self.allocate(allocated_len, amount).expect("use freshly-allocated segment"))
+        (
+            allocated_len,
+            self.allocate(allocated_len, amount)
+                .expect("use freshly-allocated segment"),
+        )
     }
 
     fn deallocate_all(&mut self) {
-        if let Some(ref mut a) = self.allocator {
+        if let Some(a) = &mut self.allocator {
+            #[cfg(feature = "alloc")]
             for seg in &self.segments {
-                a.deallocate_segment(seg.ptr, seg.capacity, seg.allocated);
+                unsafe {
+                    a.deallocate_segment(seg.ptr, seg.capacity, seg.allocated);
+                }
+            }
+
+            #[cfg(not(feature = "alloc"))]
+            if let Some(seg) = &self.segments.segment {
+                unsafe {
+                    a.deallocate_segment(seg.ptr, seg.capacity, seg.allocated);
+                }
             }
         }
     }
@@ -285,17 +410,20 @@ impl <A> BuilderArenaImplInner<A> where A: Allocator {
     }
 }
 
-impl <A> BuilderArena for BuilderArenaImpl<A> where A: Allocator {
-    fn allocate(&self, segment_id: u32, amount: WordCount32) -> Option<u32> {
-        self.inner.borrow_mut().allocate(segment_id, amount)
+impl<A> BuilderArena for BuilderArenaImpl<A>
+where
+    A: Allocator,
+{
+    fn allocate(&mut self, segment_id: u32, amount: WordCount32) -> Option<u32> {
+        self.inner.allocate(segment_id, amount)
     }
 
-    fn allocate_anywhere(&self, amount: u32) -> (SegmentId, u32) {
-        self.inner.borrow_mut().allocate_anywhere(amount)
+    fn allocate_anywhere(&mut self, amount: u32) -> (SegmentId, u32) {
+        self.inner.allocate_anywhere(amount)
     }
 
-    fn get_segment_mut(&self, id: u32) -> (*mut u8, u32) {
-        self.inner.borrow_mut().get_segment_mut(id)
+    fn get_segment_mut(&mut self, id: u32) -> (*mut u8, u32) {
+        self.inner.get_segment_mut(id)
     }
 
     fn as_reader(&self) -> &dyn ReaderArena {
@@ -303,7 +431,10 @@ impl <A> BuilderArena for BuilderArenaImpl<A> where A: Allocator {
     }
 }
 
-impl <A> Drop for BuilderArenaImplInner<A> where A: Allocator {
+impl<A> Drop for BuilderArenaImplInner<A>
+where
+    A: Allocator,
+{
     fn drop(&mut self) {
         self.deallocate_all()
     }
@@ -313,11 +444,16 @@ pub struct NullArena;
 
 impl ReaderArena for NullArena {
     fn get_segment(&self, _id: u32) -> Result<(*const u8, u32)> {
-        Err(Error::failed(String::from("tried to read from null arena")))
+        Err(Error::from_kind(ErrorKind::TriedToReadFromNullArena))
     }
 
-    fn check_offset(&self, _segment_id: u32, start: *const u8, offset_in_words: i32) -> Result<*const u8> {
-        unsafe { Ok(start.offset((offset_in_words as usize * BYTES_PER_WORD)as isize)) }
+    unsafe fn check_offset(
+        &self,
+        _segment_id: u32,
+        start: *const u8,
+        offset_in_words: i32,
+    ) -> Result<*const u8> {
+        unsafe { Ok(start.add(offset_in_words as usize * BYTES_PER_WORD)) }
     }
 
     fn contains_interval(&self, _id: u32, _start: *const u8, _size: usize) -> Result<()> {
@@ -328,24 +464,7 @@ impl ReaderArena for NullArena {
         Ok(())
     }
 
-    fn nesting_limit(&self) -> i32 { 0x7fffffff }
-}
-
-impl BuilderArena for NullArena {
-    fn allocate(&self, _segment_id: u32, _amount: WordCount32) -> Option<u32> {
-        None
-    }
-
-    fn allocate_anywhere(&self, _amount: u32) -> (SegmentId, u32) {
-        panic!("tried to allocate from a null arena")
-    }
-
-    fn get_segment_mut(&self, _id: u32) -> (*mut u8, u32) {
-        (core::ptr::null_mut(), 0)
-    }
-
-    fn as_reader(&self) -> &dyn ReaderArena {
-        self
+    fn nesting_limit(&self) -> i32 {
+        0x7fffffff
     }
 }
-
